<?php

include 'encoder.php';

/**
 * @author Alexey Kupershtokh <alexey.kupershtokh@gmail.com>
 * @url http://kupershtokh.blogspot.com/2007/12/on-phpclub.html
 */
class TTH {
	private static $tiger_hash = null;
	private static $tiger_mhash = null;

	/**
	 * Generates TTH of a string.
	 *
	 * @param string $string
	 * @return string
	 */
	public static function hashString($string) {
		// least code duplication
		return self::hashFile('data://text/plain;base64,' . base64_encode($string));
	}

	/**
	 * Generates TTH of a file.
	 *
	 * @param string $filename
	 * @return string
	 */
	public static function hashFile($filename) {
		$fp = fopen($filename, "rb");
		if($fp) {
			$i = 1;
			$hashes = array();
			while(!feof($fp)) {
				$buf = fread($fp, 1024);
				if ($buf || ($i == 1)) {
					$hashes[$i] = self::tiger("\0".$buf);
					$j = 1;
					while($i % ($j * 2) == 0) {
						$hashes[$i] = self::tiger("\1".$hashes[$i - $j].$hashes[$i]);
						unset($hashes[$i - $j]);
						$j = round($j * 2);
					}
					$i++;
				}
			}
			$k = 1;
			while($i > $k) {
				$k = round($k * 2);
			}
			for(; $i <= $k; $i++) {
				$j = 1;
				while($i % ($j * 2) == 0) {
					if(isset($hashes[$i])) {
						$hashes[$i] = self::tiger("\1".$hashes[$i - $j].$hashes[$i]);
					} elseif(isset($hashes[$i - $j])) {
						$hashes[$i] = $hashes[$i - $j];
					}
					unset($hashes[$i - $j]);
					$j = round($j * 2);
				}
			}
			fclose($fp);

			return Encoder::toBase32($hashes[$i-1]);
		}
	}

	/**
	 * Generates a tiger hash (not TTH).
	 * Automatically chooses between hash() and mhash().
	 *
	 * @param string $string
	 * @return string
	 */
	private static function tiger($string) {
		if (is_null(self::$tiger_hash)) {
			 self::$tiger_hash = function_exists("hash_algos") && in_array("tiger192,3", hash_algos());
		}
		if (self::$tiger_hash) {
			return self::tigerfix(hash("tiger192,3", $string, 1));
		}

		if (is_null(self::$tiger_mhash)) {
			self::$tiger_mhash = function_exists("mhash");
		}
		if(self::$tiger_mhash) {
			return self::tigerfix(mhash(MHASH_TIGER, $string));
		}

		trigger_error(E_USER_ERROR, "Neither tiger hash function is available.");
	}

	/**
	 * Repairs tiger hash for compatibility (byte order).
	 *
	 * @url https://bugs.php.net/bug.php?id=32463
	 * @param string $binary_hash
	 * @return string
	 */ 
	private static function tigerfix($binary_hash) {
		$data = unpack('C*', $binary_hash);
		$fixed = '';
		for($j = 0, $size = sizeof($data); $j < $size; ++$j)
			$fixed .= pack('C', $data[($j^7) + 1]);
		return $fixed;
	}
}

?>